<?php

namespace Drupal\ckeditor5_audio_plugin\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns response for CKEditor 5 audio upload adapter.
 *
 * @internal
 *   Controller classes are internal.
 */
class AudioUploadController extends ControllerBase {

  /**
   * The file system service.
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The The file url generator.
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Constructs a new CKEditor5ImageController.
   *
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file url generator.
   */
  public function __construct(FileSystemInterface $fileSystem, FileUrlGeneratorInterface $fileUrlGenerator) {
    $this->fileSystem = $fileSystem;
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('file_url_generator')
    );
  }

  /**
   * Uploads and saves an audio from a CKEditor 5 POST.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON object including the file URL
   *   or error with status 400.
   */
  public function upload(Request $request) {
    // Get file, directory, and max_size from the request.
    $file = $request->files->get('file');
    $directory = $request->get('directory', 'public://');
    $max_size = $request->get('max_size', 0);

    // Check if a file is uploaded.
    if (!$file) {
      return new JsonResponse(['error' => 'No file uploaded'], 400);
    }

    // Validate file size if a max size is provided.
    if ($max_size && $file->getSize() > (int) $max_size) {
      return new JsonResponse(['error' => 'File exceeds the maximum allowed size'], 400);
    }

    // Ensure the directory is safe and formatted correctly.
    $directory = sprintf('public://%s', trim($directory, '/'));

    // Check if the directory can be created or already exists.
    if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY)) {
      return new JsonResponse(['error' => 'Cannot prepare the directory for upload'], 400);
    }

    // Generate the file destination path and move the file.
    $destination = $directory . '/' . $file->getClientOriginalName();

    try {
      $this->fileSystem->move($file->getRealPath(), $destination);
    }
    catch (\Exception $e) {
      return new JsonResponse(['error' => 'File upload failed'], 400);
    }

    // Generate the file URL and return the response.
    $fileUrl = $this->fileUrlGenerator->generateAbsoluteString($destination);

    return new JsonResponse(['url' => $fileUrl]);
  }

}
